# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import json
import urlparse
import hashlib
import urllib

from qommon.misc import http_get_page, json_loads
from quixote import get_publisher, get_response
from quixote.html import htmltext


def has_file_validation():
    return get_publisher().get_site_option('fargo_url') is not None

def fargo_get(path):
    fargo_url = get_publisher().get_site_option('fargo_url')
    url = urlparse.urljoin(fargo_url, path)
    response, status, data, auth_header = http_get_page(url)
    if status == 200:
        return json_loads(data)
    return None

def sha256_of_upload(upload):
    return hashlib.sha256(upload.get_content()).hexdigest()

def get_document_types():
    if not has_file_validation():
        return {}
    response = fargo_get('/document-types/')
    publisher = get_publisher()
    if response.get('err') == 0:
        result = {}
        for schema in response['data']:
            d = {
                'id': schema['name'],
                'label': schema['label'],
                'fargo': True,
            }
            if 'mimetypes' in schema:
                d['mimetypes'] = shema['mimetypes']
            result[d['id']] = d

        return result
    return {}

def validation_path(filled, field, upload):
    user = filled.get_user()
    if not user:
        return None
    if not user.name_identifiers:
        return None
    if not field.document_type or not field.document_type.get('fargo'):
        return None
    name_id = user.name_identifiers[0]
    sha_256 = sha256_of_upload(upload)
    document_type = field.document_type['id']
    path = '%s/%s/%s/' % (
        urllib.quote(name_id),
        urllib.quote(sha_256),
        urllib.quote(document_type),
    )
    return path

def validate_upload(filled, field, upload):
    '''Check validation of the uploaded file with Fargo'''
    path = validation_path(filled, field, upload)
    if not path:
        return None
    response = fargo_get('metadata/' + path)
    if response is None:
        return None
    if response['err'] == 1:
        return False
    return response['data']

def get_document_type_label(document_type):
    return get_document_types().get(document_type, {}).get('label')

def validation_link(filled, field, upload):
    '''Compute link to Fargo to validate the given document'''
    path = validation_path(filled, field, upload)
    if not path:
        return ''
    get_response().add_css_include('../js/smoothness/jquery-ui-1.10.0.custom.min.css')
    get_response().add_javascript(['jquery-ui.js', 'jquery.js', 'fargo.js'])
    label = get_document_type_label(field.document_type['id'])
    fargo_url = get_publisher().get_site_option('fargo_url')
    url = urlparse.urljoin(fargo_url, 'validation/' + path)
    next_url = get_publisher().get_frontoffice_url() + '/reload-top'
    url += '?next=%s' % urllib.quote(next_url)
    title = _('Validate as a %s') % label
    return htmltext('<a data-title="%(title)s" data-width="800" '
                    'data-height="500" href="%(url)s">%(title)s</a>') % {
        'title': title,
        'url': url,
    }
